program main
  use const_and_precisions, only : r8, pi
  implicit none
  integer, parameter :: cocosout=3
  integer :: u, ierr, cocos, exp2pi, exp2piout, idum, i, j, ipsinorm
  logical :: ispsinorm
  logical :: phiccw, psiincr, qpos, phiccwout, psiincrout, qposout
  character(len=255) :: desc, eqdskfile, prffile
  character(len=*), parameter :: fmt2000='(a48,3i4)',fmt2020='(5e16.9)',fmt2022='(2i5)'
  integer :: nr, nz, nbnd, nprf
  real(r8) :: deltar, deltaz, r0, r1, zmid, rax, zax, psiax, psib, bref, ipla, &
      dummy, z1, dr, dz, dpsin, bref0, ipla0, qb0, psifact, psii, powin, pec, icd
  real(r8), dimension(:,:), allocatable :: psi2d
  real(r8), dimension(:),   allocatable :: r, z, psi1d, q, fdia, rbnd, zbnd, &
      psiprf, te, dne, zeff, cte, cne, czeff, teeq, dneeq, zeffeq, dpdv, jcd
  real(r8), external :: fspli

! === read filenames and input power from gray.data ===
  u = get_free_unit()
  open(u,file='gray.data',status= 'unknown')
  read(u,*)
  read(u,*)
  read(u,*) powin
  do i=1,13
    read(u,*)
  end do
  read(u,*) desc
  eqdskfile=trim(desc)//'.eqdsk'
  read(u,*) ipsinorm
  ispsinorm=(ipsinorm==1)
  read(u,*)
  read(u,*) desc
  prffile=trim(desc)//'.prf'
  read(u,*)
  read(u,*) dummy,dummy,cocos
  close(u)

! === read EQDSK ===
  u = get_free_unit()
  open(unit=u,file=eqdskfile,status='OLD',action='READ',iostat=ierr)
  if(ierr/=0) then
    write(*,*) 'Cannot open file '//trim(eqdskfile)
    stop
  end if

! read grid size and allocate arrays
  read (u,fmt2000) desc,idum,nr,nz
  allocate(psi2d(nr,nz), r(nr), z(nz), psi1d(nr), q(nr), fdia(nr), stat=ierr)
  if (ierr/=0) then
    close(u)
    call free_allocs
    write(*,*) 'cannot allocate arrays for equilibrium data'
    stop
  end if
! read 0D fields
  read (u,fmt2020) deltar, deltaz, r0, r1, zmid
  read (u,fmt2020) rax, zax, psiax, psib, bref
  read (u,fmt2020) ipla, (dummy,i=1,4)
  read (u,fmt2020) (dummy,i=1,5)
! read 1D-2D fields
  read (u,fmt2020) (fdia(i),i=1,nr)
  read (u,fmt2020) (dummy,i=1,nr)
  read (u,fmt2020) (dummy,i=1,nr)
  read (u,fmt2020) (dummy,i=1,nr)
  read (u,fmt2020) ((psi2d(i,j),i=1,nr),j=1,nz)
  read (u,fmt2020) (q(i),i=1,nr)
! read boundary size and allocate arrays
  read (u,fmt2022) nbnd
  if (nbnd>0) then
    allocate(rbnd(nbnd), zbnd(nbnd), stat=ierr)
    if (ierr/=0) then
      close(u)
      call free_allocs
      write(*,*) 'cannot allocate arrays for boundary data'
      stop
    end if
! read boundary shape
    read (u,fmt2020) (rbnd(i), zbnd(i),i=1,nbnd)
  end if
  close(u)

! normalize psi2d
  if (.not.ispsinorm) psi2d = (psi2d - psiax)/(psib - psiax)

! interpret cocos numbers
  call decode_cocos(cocos,exp2pi,phiccw,psiincr,qpos)
  call decode_cocos(cocosout,exp2piout,phiccwout,psiincrout,qposout)

! check sign consistency
  ipla0=ipla
  bref0=bref
  if (psiincr) then
    ipla=sign(ipla,psib - psiax)
  else
    ipla=sign(ipla,psiax - psib)
  end if
  bref=sign(bref,fdia(nr))
  if (ipla/ipla0<0 .or. bref/bref0<0) then
    write(*,*) 'Warning: sign inconsistency in Ipla/psi or Bref/Fdia'
  end if
  qb0=q(nr)
  if (qpos) then
    q=sign(q,ipla*bref)
  else
    q=sign(q,-ipla*bref)
  end if
  if (q(nr)/qb0<0) then
    write(*,*) 'Warning: sign inconsistency found among q, Ipla and Bref'
  end if

! convert cocosin to cocosout
  if (phiccw.neqv.phiccwout) then
!   opposite direction of toroidal angle phi in cocosin and cocosout
    bref=-bref
    ipla=-ipla
    fdia=-fdia
  end if
  if ((phiccw.eqv.phiccwout) .neqv. (psiincr.eqv.psiincrout)) then
!   psi and Ip signs don't change accordingly
    psib=-psib
    psiax=-psiax
  end if
  if (qpos .neqv. qposout) q=-q
!   q has opposite sign for given sign of Bphi*Ip
  if (exp2pi/=exp2piout) then
!   convert Wb to Wb/rad or viceversa
    psifact=(2._r8*pi)**(exp2piout-exp2pi)
    psib=psib*psifact
    psiax=psiax*psifact
  end if

! fill equi-spaced R, z, psi arrays
  z1 = zmid - deltaz/2._r8
  dr = deltar/(nr-1)
  dz = deltaz/(nz-1)
  dpsin = 1._r8/(nr-1)
  do i=1,nr
    r(i) = r1 + (i-1)*dr
    psi1d(i) = (i-1)*dpsin
  end do
  do j=1,nz
    z(j) = z1 + (j-1)*dz
  end do
   
! === read profiles ===
  u = get_free_unit()
  open(unit=u,file=prffile,status='OLD',action='READ',iostat=ierr)
  if(ierr/=0) then
    write(*,*) 'Cannot open file '//trim(prffile)
    stop
  end if
  read(u,*) nprf
  if (nprf>0) then
    allocate(psiprf(nprf),te(nprf),dne(nprf),zeff(nprf), stat=ierr)
    if (ierr==0) allocate(cte(4*nprf),cne(4*nprf),czeff(4*nprf), stat=ierr)
    if (ierr/=0) then
      close(u)
      call free_allocs
      write(*,*) 'cannot allocate arrays for input 1d profiles'
      stop
    end if
    do i=1,nprf
      read(u,*) psiprf(i),te(i),dne(i),zeff(i)
    end do
  else
    write(*,*) 'no data for 1d profiles'
    stop
  end if
  close(u)

! spline interpolation for resampling on uniform grid
  call difcsg(psiprf,te  ,nprf,0,cte  ,ierr)
  if (ierr==0) call difcsg(psiprf,dne ,nprf,0,cne  ,ierr)
  if (ierr==0) call difcsg(psiprf,zeff,nprf,0,czeff,ierr)
  if (ierr/=0) then
    call free_allocs
    write(*,*) 'error in input 1d profiles interpolation'
    stop
  end if
  allocate(teeq(nr),dneeq(nr),zeffeq(nr),dpdv(nr),jcd(nr), stat=ierr)
  if (ierr/=0) then
    call free_allocs
    write(*,*) 'cannot allocate arrays for resampled and output 1d profiles'
    stop
  end if
  do i=1,nr
    psii=psi1d(i)
    call vlocate(psiprf,nprf,psii,j)
    j=max(1,min(j,nprf-1))
    dpsin=psii-psiprf(j)
    teeq(i)  =fspli(cte,  nprf,j,dpsin)
    dneeq(i) =fspli(cne,  nprf,j,dpsin)
    zeffeq(i)=fspli(czeff,nprf,j,dpsin)
  end do

! convert keV to eV, 10^19 m^-3 to m^-3, and MW to W
  teeq=teeq*1.e3_r8
  dneeq=dneeq*1.e19_r8
  powin=powin*1.e6_r8

! === call GRAY subroutine ===
  call gray_main(1, nr, nz, r, z, psi2d, psiax, psib, &
    rax, zax, nbnd, rbnd, zbnd, nr, psi1d, fdia, teeq, dneeq, zeffeq, q, &
    powin, dpdv, jcd, pec, icd, ierr)

  call free_allocs
  
contains

  function get_free_unit(umin,umax) result(i)
    implicit none
    integer :: i
    integer, intent(in), optional :: umin, umax
    integer, parameter :: max_allowed = 99
    integer :: ierr, iend
    logical :: ex, op

    if (present(umin)) then
      i = max(0,umin)    ! start searching from unit min
    else
      i = 0
    end if
    if (present(umax)) then
      iend = min(max(0,umax),max_allowed)
    else
      iend = max_allowed
    end if
    do
      if (i>iend) then
        i=-1  ! no free units found
        exit
      end if
      inquire(unit=i,exist=ex,opened=op,iostat=ierr)
      if (ierr/=0) then
        i=-2  ! cannot inquire unit i
        exit
      end if
      if (ex.and..not.op) exit  ! unit i exists and is not open
      i = i + 1
    end do
  end function get_free_unit

  subroutine decode_cocos(cocos,exp2pi,phiccw,psiincr,qpos)
    implicit none
    integer, intent(in) :: cocos
    integer, intent(out) :: exp2pi
    logical, intent(out) :: phiccw,psiincr,qpos
    integer :: cocosm10,cocosm4

    cocosm10=mod(cocos,10)
    cocosm4=mod(cocosm10,4)
!  cocos>10 psi in Wb, cocos<10 psi in Wb/rad
    exp2pi=(cocos-cocosm10)/10
!  cocos mod 10 = 1,3,5,7: toroidal angle phi increasing CCW
    phiccw=(mod(cocosm10,2)==1)
!  cocos mod 10 = 1,2,5,6: psi increasing with positive Ip
    psiincr=(cocosm4==1 .or. cocosm4==2)
!  cocos mod 10 = 1,2,7,8: q positive for positive Bphi*Ip
    qpos=(cocosm10<3 .or. cocosm10>6)
  end subroutine decode_cocos

  subroutine free_allocs
    implicit none
    if(allocated(psi2d))  deallocate(psi2d)
    if(allocated(r))      deallocate(r)
    if(allocated(z))      deallocate(z)
    if(allocated(psi1d))  deallocate(psi1d)
    if(allocated(q))      deallocate(q)
    if(allocated(fdia))   deallocate(fdia)
    if(allocated(rbnd))   deallocate(rbnd)
    if(allocated(zbnd))   deallocate(zbnd)
    if(allocated(psiprf)) deallocate(psiprf)
    if(allocated(te))     deallocate(te)
    if(allocated(dne))    deallocate(dne)
    if(allocated(zeff))   deallocate(zeff)
    if(allocated(cte))    deallocate(cte)
    if(allocated(cne))    deallocate(cne)
    if(allocated(czeff))  deallocate(czeff)
    if(allocated(teeq))   deallocate(teeq)
    if(allocated(dneeq))  deallocate(dneeq)
    if(allocated(zeffeq)) deallocate(zeffeq)
    if(allocated(dpdv))  deallocate(dpdv)
    if(allocated(jcd)) deallocate(jcd)
  end subroutine free_allocs

end program main